@@ -37,8 +37,7 @@
  * OF SUCH DAMAGE.
  */
 
-#include <stdlib.h>
-#include <string.h>
+#define LIBSSH2_CRYPTO_ENGINE libssh2_mbedtls
 
 #include <mbedtls/platform.h>
 #include <mbedtls/md.h>
@@ -63,14 +62,25 @@
 #define LIBSSH2_HMAC_SHA256     1
 #define LIBSSH2_HMAC_SHA512     1
 
-#define LIBSSH2_AES             1
+#define LIBSSH2_AES_CBC         1
 #define LIBSSH2_AES_CTR         1
-#define LIBSSH2_BLOWFISH        1
-#define LIBSSH2_RC4             1
+#define LIBSSH2_AES_GCM         0
+#ifdef MBEDTLS_CIPHER_BLOWFISH_CBC
+# define LIBSSH2_BLOWFISH       1
+#else
+# define LIBSSH2_BLOWFISH       0
+#endif
+#ifdef MBEDTLS_CIPHER_ARC4_128
+# define LIBSSH2_RC4            1
+#else
+# define LIBSSH2_RC4            0
+#endif
 #define LIBSSH2_CAST            0
 #define LIBSSH2_3DES            1
 
 #define LIBSSH2_RSA             1
+#define LIBSSH2_RSA_SHA1        1
+#define LIBSSH2_RSA_SHA2        1
 #define LIBSSH2_DSA             0
 #ifdef MBEDTLS_ECDSA_C
 # define LIBSSH2_ECDSA          1
@@ -94,12 +104,12 @@
  */
 
 #define libssh2_crypto_init() \
-  _libssh2_mbedtls_init()
+    _libssh2_mbedtls_init()
 #define libssh2_crypto_exit() \
-  _libssh2_mbedtls_free()
+    _libssh2_mbedtls_free()
 
 #define _libssh2_random(buf, len) \
-  _libssh2_mbedtls_random(buf, len)
+    _libssh2_mbedtls_random(buf, len)
 
 #define libssh2_prepare_iovec(vec, len)  /* Empty. */
 
@@ -113,24 +123,24 @@
 
 #define libssh2_hmac_ctx_init(ctx)
 #define libssh2_hmac_cleanup(pctx) \
-  mbedtls_md_free(pctx)
+    mbedtls_md_free(pctx)
 #define libssh2_hmac_update(ctx, data, datalen) \
-  mbedtls_md_hmac_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_hmac_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_hmac_final(ctx, hash) \
-  mbedtls_md_hmac_finish(&ctx, hash)
+    mbedtls_md_hmac_finish(&ctx, hash)
 
 #define libssh2_hmac_sha1_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA1, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA1, key, keylen)
 #define libssh2_hmac_md5_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_MD5, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_MD5, key, keylen)
 #define libssh2_hmac_ripemd160_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_RIPEMD160, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_RIPEMD160, key, keylen)
 #define libssh2_hmac_sha256_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA256, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA256, key, keylen)
 #define libssh2_hmac_sha384_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA384, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA384, key, keylen)
 #define libssh2_hmac_sha512_init(pctx, key, keylen) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA512, key, keylen)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA512, key, keylen)
 
 
 /*******************************************************************/
@@ -141,13 +151,14 @@
 #define libssh2_sha1_ctx      mbedtls_md_context_t
 
 #define libssh2_sha1_init(pctx) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA1, NULL, 0)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA1, NULL, 0)
 #define libssh2_sha1_update(ctx, data, datalen) \
-  mbedtls_md_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_sha1_final(ctx, hash) \
-  _libssh2_mbedtls_hash_final(&ctx, hash)
+    _libssh2_mbedtls_hash_final(&ctx, hash)
 #define libssh2_sha1(data, datalen, hash) \
-  _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA1, hash)
+    _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA1, hash)
+
 
 /*******************************************************************/
 /*
@@ -157,13 +168,13 @@
 #define libssh2_sha256_ctx      mbedtls_md_context_t
 
 #define libssh2_sha256_init(pctx) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA256, NULL, 0)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA256, NULL, 0)
 #define libssh2_sha256_update(ctx, data, datalen) \
-  mbedtls_md_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_sha256_final(ctx, hash) \
-  _libssh2_mbedtls_hash_final(&ctx, hash)
+    _libssh2_mbedtls_hash_final(&ctx, hash)
 #define libssh2_sha256(data, datalen, hash) \
-  _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA256, hash)
+    _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA256, hash)
 
 
 /*******************************************************************/
@@ -174,13 +185,13 @@
 #define libssh2_sha384_ctx      mbedtls_md_context_t
 
 #define libssh2_sha384_init(pctx) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA384, NULL, 0)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA384, NULL, 0)
 #define libssh2_sha384_update(ctx, data, datalen) \
-  mbedtls_md_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_sha384_final(ctx, hash) \
-  _libssh2_mbedtls_hash_final(&ctx, hash)
+    _libssh2_mbedtls_hash_final(&ctx, hash)
 #define libssh2_sha384(data, datalen, hash) \
-  _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA384, hash)
+    _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA384, hash)
 
 
 /*******************************************************************/
@@ -191,13 +202,13 @@
 #define libssh2_sha512_ctx      mbedtls_md_context_t
 
 #define libssh2_sha512_init(pctx) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA512, NULL, 0)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_SHA512, NULL, 0)
 #define libssh2_sha512_update(ctx, data, datalen) \
-  mbedtls_md_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_sha512_final(ctx, hash) \
-  _libssh2_mbedtls_hash_final(&ctx, hash)
+    _libssh2_mbedtls_hash_final(&ctx, hash)
 #define libssh2_sha512(data, datalen, hash) \
-  _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA512, hash)
+    _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_SHA512, hash)
 
 
 /*******************************************************************/
@@ -208,13 +219,13 @@
 #define libssh2_md5_ctx      mbedtls_md_context_t
 
 #define libssh2_md5_init(pctx) \
-  _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_MD5, NULL, 0)
+    _libssh2_mbedtls_hash_init(pctx, MBEDTLS_MD_MD5, NULL, 0)
 #define libssh2_md5_update(ctx, data, datalen) \
-  mbedtls_md_update(&ctx, (unsigned char *) data, datalen)
+    mbedtls_md_update(&ctx, (const unsigned char *) data, datalen)
 #define libssh2_md5_final(ctx, hash) \
-  _libssh2_mbedtls_hash_final(&ctx, hash)
+    _libssh2_mbedtls_hash_final(&ctx, hash)
 #define libssh2_md5(data, datalen, hash) \
-  _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_MD5, hash)
+    _libssh2_mbedtls_hash(data, datalen, MBEDTLS_MD_MD5, hash)
 
 
 /*******************************************************************/
@@ -227,26 +238,33 @@
 #define _libssh2_rsa_new(rsactx, e, e_len, n, n_len, \
                          d, d_len, p, p_len, q, q_len, \
                          e1, e1_len, e2, e2_len, c, c_len) \
-  _libssh2_mbedtls_rsa_new(rsactx, e, e_len, n, n_len, \
-                          d, d_len, p, p_len, q, q_len, \
-                          e1, e1_len, e2, e2_len, c, c_len)
+    _libssh2_mbedtls_rsa_new(rsactx, e, e_len, n, n_len, \
+                             d, d_len, p, p_len, q, q_len, \
+                             e1, e1_len, e2, e2_len, c, c_len)
 
 #define _libssh2_rsa_new_private(rsactx, s, filename, passphrase) \
-  _libssh2_mbedtls_rsa_new_private(rsactx, s, filename, passphrase)
+    _libssh2_mbedtls_rsa_new_private(rsactx, s, filename, passphrase)
 
 #define _libssh2_rsa_new_private_frommemory(rsactx, s, filedata, \
                                             filedata_len, passphrase) \
-  _libssh2_mbedtls_rsa_new_private_frommemory(rsactx, s, filedata, \
-                                             filedata_len, passphrase)
+    _libssh2_mbedtls_rsa_new_private_frommemory(rsactx, s, filedata, \
+                                                filedata_len, passphrase)
 
 #define _libssh2_rsa_sha1_sign(s, rsactx, hash, hash_len, sig, sig_len) \
-  _libssh2_mbedtls_rsa_sha1_sign(s, rsactx, hash, hash_len, sig, sig_len)
+    _libssh2_mbedtls_rsa_sha1_sign(s, rsactx, hash, hash_len, sig, sig_len)
+
+#define _libssh2_rsa_sha2_sign(s, rsactx, hash, hash_len, sig, sig_len) \
+    _libssh2_mbedtls_rsa_sha2_sign(s, rsactx, hash, hash_len, sig, sig_len)
+
 
 #define _libssh2_rsa_sha1_verify(rsactx, sig, sig_len, m, m_len) \
-  _libssh2_mbedtls_rsa_sha1_verify(rsactx, sig, sig_len, m, m_len)
+    _libssh2_mbedtls_rsa_sha1_verify(rsactx, sig, sig_len, m, m_len)
+
+#define _libssh2_rsa_sha2_verify(rsactx, hash_len, sig, sig_len, m, m_len) \
+    _libssh2_mbedtls_rsa_sha2_verify(rsactx, hash_len, sig, sig_len, m, m_len)
 
 #define _libssh2_rsa_free(rsactx) \
-  _libssh2_mbedtls_rsa_free(rsactx)
+    _libssh2_mbedtls_rsa_free(rsactx)
 
 
 /*******************************************************************/
@@ -291,34 +309,34 @@ typedef enum {
 
 #define _libssh2_ecdsa_create_key(session, privkey, pubkey_octal, \
                                   pubkey_octal_len, curve) \
-  _libssh2_mbedtls_ecdsa_create_key(session, privkey, pubkey_octal, \
-                                    pubkey_octal_len, curve)
+    _libssh2_mbedtls_ecdsa_create_key(session, privkey, pubkey_octal, \
+                                      pubkey_octal_len, curve)
 
 #define _libssh2_ecdsa_curve_name_with_octal_new(ctx, k, k_len, curve) \
-  _libssh2_mbedtls_ecdsa_curve_name_with_octal_new(ctx, k, k_len, curve)
+    _libssh2_mbedtls_ecdsa_curve_name_with_octal_new(ctx, k, k_len, curve)
 
 #define _libssh2_ecdh_gen_k(k, privkey, server_pubkey, server_pubkey_len) \
-  _libssh2_mbedtls_ecdh_gen_k(k, privkey, server_pubkey, server_pubkey_len)
+    _libssh2_mbedtls_ecdh_gen_k(k, privkey, server_pubkey, server_pubkey_len)
 
 #define _libssh2_ecdsa_verify(ctx, r, r_len, s, s_len, m, m_len) \
-  _libssh2_mbedtls_ecdsa_verify(ctx, r, r_len, s, s_len, m, m_len)
+    _libssh2_mbedtls_ecdsa_verify(ctx, r, r_len, s, s_len, m, m_len)
 
 #define _libssh2_ecdsa_new_private(ctx, session, filename, passphrase) \
-  _libssh2_mbedtls_ecdsa_new_private(ctx, session, filename, passphrase)
+    _libssh2_mbedtls_ecdsa_new_private(ctx, session, filename, passphrase)
 
 #define _libssh2_ecdsa_new_private_frommemory(ctx, session, filedata, \
                                               filedata_len, passphrase) \
-  _libssh2_mbedtls_ecdsa_new_private_frommemory(ctx, session, filedata, \
-                                                filedata_len, passphrase)
+    _libssh2_mbedtls_ecdsa_new_private_frommemory(ctx, session, filedata, \
+                                                  filedata_len, passphrase)
 
 #define _libssh2_ecdsa_sign(session, ctx, hash, hash_len, sign, sign_len) \
-  _libssh2_mbedtls_ecdsa_sign(session, ctx, hash, hash_len, sign, sign_len)
+    _libssh2_mbedtls_ecdsa_sign(session, ctx, hash, hash_len, sign, sign_len)
 
 #define _libssh2_ecdsa_get_curve_type(ctx) \
-  _libssh2_mbedtls_ecdsa_get_curve_type(ctx)
+    _libssh2_mbedtls_ecdsa_get_curve_type(ctx)
 
 #define _libssh2_ecdsa_free(ctx) \
-  _libssh2_mbedtls_ecdsa_free(ctx)
+    _libssh2_mbedtls_ecdsa_free(ctx)
 
 #endif /* LIBSSH2_ECDSA */
 
@@ -329,11 +347,15 @@ typedef enum {
  */
 
 #define _libssh2_pub_priv_keyfile(s, m, m_len, p, p_len, pk, pw) \
-  _libssh2_mbedtls_pub_priv_keyfile(s, m, m_len, p, p_len, pk, pw)
+    _libssh2_mbedtls_pub_priv_keyfile(s, m, m_len, p, p_len, pk, pw)
 #define _libssh2_pub_priv_keyfilememory(s, m, m_len, p, p_len, \
-                                                     pk, pk_len, pw) \
-  _libssh2_mbedtls_pub_priv_keyfilememory(s, m, m_len, p, p_len, \
-                                                      pk, pk_len, pw)
+                                        pk, pk_len, pw) \
+    _libssh2_mbedtls_pub_priv_keyfilememory(s, m, m_len, p, p_len, \
+                                            pk, pk_len, pw)
+#define _libssh2_sk_pub_keyfilememory(s, m, m_len, p, p_len, alg, app, \
+                                      f, kh, kh_len, pk, pk_len, pw) \
+    _libssh2_mbedtls_sk_pub_keyfilememory(s, m, m_len, p, p_len, alg, app, \
+                                          f, kh, kh_len, pk, pk_len, pw)
 
 
 /*******************************************************************/
@@ -351,9 +373,12 @@ typedef enum {
 #define _libssh2_cipher_aes256    MBEDTLS_CIPHER_AES_256_CBC
 #define _libssh2_cipher_aes192    MBEDTLS_CIPHER_AES_192_CBC
 #define _libssh2_cipher_aes128    MBEDTLS_CIPHER_AES_128_CBC
+#ifdef MBEDTLS_CIPHER_BLOWFISH_CBC
 #define _libssh2_cipher_blowfish  MBEDTLS_CIPHER_BLOWFISH_CBC
+#endif
+#ifdef MBEDTLS_CIPHER_ARC4_128
 #define _libssh2_cipher_arcfour   MBEDTLS_CIPHER_ARC4_128
-#define _libssh2_cipher_cast5     MBEDTLS_CIPHER_NULL
+#endif
 #define _libssh2_cipher_3des      MBEDTLS_CIPHER_DES_EDE3_CBC
 
 
@@ -363,11 +388,11 @@ typedef enum {
  */
 
 #define _libssh2_cipher_init(ctx, type, iv, secret, encrypt) \
-  _libssh2_mbedtls_cipher_init(ctx, type, iv, secret, encrypt)
-#define _libssh2_cipher_crypt(ctx, type, encrypt, block, blocklen) \
-  _libssh2_mbedtls_cipher_crypt(ctx, type, encrypt, block, blocklen)
+    _libssh2_mbedtls_cipher_init(ctx, type, iv, secret, encrypt)
+#define _libssh2_cipher_crypt(ctx, type, encrypt, block, blocklen, fl) \
+    _libssh2_mbedtls_cipher_crypt(ctx, type, encrypt, block, blocklen, fl)
 #define _libssh2_cipher_dtor(ctx) \
-  _libssh2_mbedtls_cipher_dtor(ctx)
+    _libssh2_mbedtls_cipher_dtor(ctx)
 
 
 /*******************************************************************/
@@ -382,21 +407,21 @@ typedef enum {
 #define _libssh2_bn mbedtls_mpi
 
 #define _libssh2_bn_init() \
-  _libssh2_mbedtls_bignum_init()
+    _libssh2_mbedtls_bignum_init()
 #define _libssh2_bn_init_from_bin() \
-  _libssh2_mbedtls_bignum_init()
+    _libssh2_mbedtls_bignum_init()
 #define _libssh2_bn_set_word(bn, word) \
-  mbedtls_mpi_lset(bn, word)
+    mbedtls_mpi_lset(bn, word)
 #define _libssh2_bn_from_bin(bn, len, bin) \
-  mbedtls_mpi_read_binary(bn, bin, len)
+    mbedtls_mpi_read_binary(bn, bin, len)
 #define _libssh2_bn_to_bin(bn, bin) \
-  mbedtls_mpi_write_binary(bn, bin, mbedtls_mpi_size(bn))
+    mbedtls_mpi_write_binary(bn, bin, mbedtls_mpi_size(bn))
 #define _libssh2_bn_bytes(bn) \
-  mbedtls_mpi_size(bn)
+    mbedtls_mpi_size(bn)
 #define _libssh2_bn_bits(bn) \
-  mbedtls_mpi_bitlen(bn)
+    mbedtls_mpi_bitlen(bn)
 #define _libssh2_bn_free(bn) \
-  _libssh2_mbedtls_bignum_free(bn)
+    _libssh2_mbedtls_bignum_free(bn)
 
 
 /*******************************************************************/
@@ -404,12 +429,20 @@ typedef enum {
  * mbedTLS backend: Diffie-Hellman support.
  */
 
+/* Default generate and safe prime sizes for
+   diffie-hellman-group-exchange-sha1 */
+#define LIBSSH2_DH_GEX_MINGROUP     2048
+#define LIBSSH2_DH_GEX_OPTGROUP     4096
+#define LIBSSH2_DH_GEX_MAXGROUP     8192
+
+#define LIBSSH2_DH_MAX_MODULUS_BITS 16384
+
 #define _libssh2_dh_ctx mbedtls_mpi *
 #define libssh2_dh_init(dhctx) _libssh2_dh_init(dhctx)
 #define libssh2_dh_key_pair(dhctx, public, g, p, group_order, bnctx) \
-        _libssh2_dh_key_pair(dhctx, public, g, p, group_order)
+    _libssh2_dh_key_pair(dhctx, public, g, p, group_order)
 #define libssh2_dh_secret(dhctx, secret, f, p, bnctx) \
-        _libssh2_dh_secret(dhctx, secret, f, p)
+    _libssh2_dh_secret(dhctx, secret, f, p)
 #define libssh2_dh_dtor(dhctx) _libssh2_dh_dtor(dhctx)
 
 
@@ -425,27 +458,27 @@ void
 _libssh2_mbedtls_free(void);
 
 int
-_libssh2_mbedtls_random(unsigned char *buf, int len);
+_libssh2_mbedtls_random(unsigned char *buf, size_t len);
 
 int
 _libssh2_mbedtls_cipher_init(_libssh2_cipher_ctx *ctx,
-                            _libssh2_cipher_type(type),
-                            unsigned char *iv,
-                            unsigned char *secret,
-                            int encrypt);
+                             _libssh2_cipher_type(type),
+                             unsigned char *iv,
+                             unsigned char *secret,
+                             int encrypt);
 int
 _libssh2_mbedtls_cipher_crypt(_libssh2_cipher_ctx *ctx,
-                             _libssh2_cipher_type(type),
-                             int encrypt,
-                             unsigned char *block,
-                             size_t blocklen);
+                              _libssh2_cipher_type(type),
+                              int encrypt,
+                              unsigned char *block,
+                              size_t blocklen, int firstlast);
 void
 _libssh2_mbedtls_cipher_dtor(_libssh2_cipher_ctx *ctx);
 
 int
 _libssh2_mbedtls_hash_init(mbedtls_md_context_t *ctx,
-                          mbedtls_md_type_t mdtype,
-                          const unsigned char *key, unsigned long keylen);
+                           mbedtls_md_type_t mdtype,
+                           const unsigned char *key, unsigned long keylen);
 
 int
 _libssh2_mbedtls_hash_final(mbedtls_md_context_t *ctx, unsigned char *hash);
@@ -461,68 +494,81 @@ _libssh2_mbedtls_bignum_free(_libssh2_bn *bn);
 
 int
 _libssh2_mbedtls_rsa_new(libssh2_rsa_ctx **rsa,
-                        const unsigned char *edata,
-                        unsigned long elen,
-                        const unsigned char *ndata,
-                        unsigned long nlen,
-                        const unsigned char *ddata,
-                        unsigned long dlen,
-                        const unsigned char *pdata,
-                        unsigned long plen,
-                        const unsigned char *qdata,
-                        unsigned long qlen,
-                        const unsigned char *e1data,
-                        unsigned long e1len,
-                        const unsigned char *e2data,
-                        unsigned long e2len,
-                        const unsigned char *coeffdata,
-                        unsigned long coefflen);
+                         const unsigned char *edata,
+                         unsigned long elen,
+                         const unsigned char *ndata,
+                         unsigned long nlen,
+                         const unsigned char *ddata,
+                         unsigned long dlen,
+                         const unsigned char *pdata,
+                         unsigned long plen,
+                         const unsigned char *qdata,
+                         unsigned long qlen,
+                         const unsigned char *e1data,
+                         unsigned long e1len,
+                         const unsigned char *e2data,
+                         unsigned long e2len,
+                         const unsigned char *coeffdata,
+                         unsigned long coefflen);
 
 int
 _libssh2_mbedtls_rsa_new_private(libssh2_rsa_ctx **rsa,
-                                LIBSSH2_SESSION *session,
-                                const char *filename,
-                                const unsigned char *passphrase);
+                                 LIBSSH2_SESSION *session,
+                                 const char *filename,
+                                 const unsigned char *passphrase);
 
 int
 _libssh2_mbedtls_rsa_new_private_frommemory(libssh2_rsa_ctx **rsa,
-                                           LIBSSH2_SESSION *session,
-                                           const char *filedata,
-                                           size_t filedata_len,
-                                           unsigned const char *passphrase);
+                                            LIBSSH2_SESSION *session,
+                                            const char *filedata,
+                                            size_t filedata_len,
+                                            unsigned const char *passphrase);
 int
 _libssh2_mbedtls_rsa_sha1_verify(libssh2_rsa_ctx *rsa,
-                                const unsigned char *sig,
-                                unsigned long sig_len,
-                                const unsigned char *m,
-                                unsigned long m_len);
+                                 const unsigned char *sig,
+                                 unsigned long sig_len,
+                                 const unsigned char *m,
+                                 unsigned long m_len);
 int
 _libssh2_mbedtls_rsa_sha1_sign(LIBSSH2_SESSION *session,
-                              libssh2_rsa_ctx *rsa,
-                              const unsigned char *hash,
-                              size_t hash_len,
-                              unsigned char **signature,
-                              size_t *signature_len);
+                               libssh2_rsa_ctx *rsa,
+                               const unsigned char *hash,
+                               size_t hash_len,
+                               unsigned char **signature,
+                               size_t *signature_len);
+int
+_libssh2_mbedtls_rsa_sha2_verify(libssh2_rsa_ctx * rsactx,
+                                 size_t hash_len,
+                                 const unsigned char *sig,
+                                 unsigned long sig_len,
+                                 const unsigned char *m, unsigned long m_len);
+int
+_libssh2_mbedtls_rsa_sha2_sign(LIBSSH2_SESSION *session,
+                               libssh2_rsa_ctx *rsa,
+                               const unsigned char *hash,
+                               size_t hash_len,
+                               unsigned char **signature,
+                               size_t *signature_len);
 void
 _libssh2_mbedtls_rsa_free(libssh2_rsa_ctx *rsa);
 
 int
 _libssh2_mbedtls_pub_priv_keyfile(LIBSSH2_SESSION *session,
-                                 unsigned char **method,
-                                 size_t *method_len,
-                                 unsigned char **pubkeydata,
-                                 size_t *pubkeydata_len,
-                                 const char *privatekey,
-                                 const char *passphrase);
+                                  unsigned char **method,
+                                  size_t *method_len,
+                                  unsigned char **pubkeydata,
+                                  size_t *pubkeydata_len,
+                                  const char *privatekey,
+                                  const char *passphrase);
 int
 _libssh2_mbedtls_pub_priv_keyfilememory(LIBSSH2_SESSION *session,
-                                       unsigned char **method,
-                                       size_t *method_len,
-                                       unsigned char **pubkeydata,
-                                       size_t *pubkeydata_len,
-                                       const char *privatekeydata,
-                                       size_t privatekeydata_len,
-                                       const char *passphrase);
+                                        unsigned char **method,
+                                        size_t *method_len,
+                                        unsigned char **pubkeydata,
+                                        size_t *pubkeydata_len,
+                                        const char *privatekeydata,
+                                        size_t privatekeydata_len,
+                                        const char *passphrase);
 #if LIBSSH2_ECDSA
 int
 _libssh2_mbedtls_ecdsa_create_key(LIBSSH2_SESSION *session,
@@ -547,9 +593,9 @@ _libssh2_mbedtls_ecdsa_verify(libssh2_ecdsa_ctx *ctx,
                               const unsigned char *m, size_t m_len);
 int
 _libssh2_mbedtls_ecdsa_new_private(libssh2_ecdsa_ctx **ctx,
-                                  LIBSSH2_SESSION *session,
-                                  const char *filename,
-                                  const unsigned char *passphrase);
+                                   LIBSSH2_SESSION *session,
+                                   const char *filename,
+                                   const unsigned char *passphrase);
 int
 _libssh2_mbedtls_ecdsa_new_private_frommemory(libssh2_ecdsa_ctx **ctx,
                                               LIBSSH2_SESSION *session,
@@ -572,14 +618,16 @@ void
 _libssh2_mbedtls_ecdsa_free(libssh2_ecdsa_ctx *ctx);
 #endif /* LIBSSH2_ECDSA */
 
+extern void
+_libssh2_init_aes_ctr(void);
 extern void
 _libssh2_dh_init(_libssh2_dh_ctx *dhctx);
 extern int
 _libssh2_dh_key_pair(_libssh2_dh_ctx *dhctx, _libssh2_bn *public,
-                    _libssh2_bn *g, _libssh2_bn *p, int group_order);
+                     _libssh2_bn *g, _libssh2_bn *p, int group_order);
 extern int
 _libssh2_dh_secret(_libssh2_dh_ctx *dhctx, _libssh2_bn *secret,
-                  _libssh2_bn *f, _libssh2_bn *p);
+                   _libssh2_bn *f, _libssh2_bn *p);
 extern void
 _libssh2_dh_dtor(_libssh2_dh_ctx *dhctx);
 
