@@ -1,5 +1,5 @@
-#ifndef LIBSSH2_SESSION_H
-#define LIBSSH2_SESSION_H
+#ifndef __LIBSSH2_SESSION_H
+#define __LIBSSH2_SESSION_H
 /* Copyright (c) 2004-2007 Sara Golemon <sarag@libssh2.org>
  * Copyright (c) 2009-2010 by Daniel Stenberg
  * Copyright (c) 2010 Simon Josefsson <simon@josefsson.org>
@@ -41,27 +41,26 @@
 
 /* Conveniance-macros to allow code like this;
 
-   int rc = BLOCK_ADJUST(rc, session, session_startup(session, sock) );
+   int rc = BLOCK_ADJUST(rc, session, session_startup(session, sock));
 
-   int rc = BLOCK_ADJUST_ERRNO(ptr, session, session_startup(session, sock) );
+   int rc = BLOCK_ADJUST_ERRNO(ptr, session, session_startup(session, sock));
 
-   The point of course being to make sure that while in non-blocking mode
-   these always return no matter what the return code is, but in blocking mode
-   it blocks if EAGAIN is the reason for the return from the underlying
-   function.
+   The point being to make sure that while in non-blocking mode these always
+   return no matter what the return code is, but in blocking mode it blocks
+   if EAGAIN is the reason for the return from the underlying function.
 
 */
-#define BLOCK_ADJUST(rc,sess,x) \
+#define BLOCK_ADJUST(rc, sess, x) \
     do { \
-       time_t entry_time = time (NULL); \
-       do { \
-          rc = x; \
-          /* the order of the check below is important to properly deal with \
-             the case when the 'sess' is freed */ \
-          if((rc != LIBSSH2_ERROR_EAGAIN) || !sess->api_block_mode) \
-              break; \
-          rc = _libssh2_wait_socket(sess, entry_time);  \
-       } while(!rc);   \
+        time_t entry_time = time(NULL); \
+        do { \
+            rc = x; \
+            /* the order of the check below is important to properly \
+               deal with the case when the 'sess' is freed */ \
+            if((rc != LIBSSH2_ERROR_EAGAIN) || !sess->api_block_mode) \
+                break; \
+            rc = _libssh2_wait_socket(sess, entry_time);  \
+        } while(!rc);   \
     } while(0)
 
 /*
@@ -70,17 +69,17 @@
  * immediately. If the API is blocking and we get a NULL we check the errno
  * and *only* if that is EAGAIN we loop and wait for socket action.
  */
-#define BLOCK_ADJUST_ERRNO(ptr,sess,x) \
+#define BLOCK_ADJUST_ERRNO(ptr, sess, x) \
     do { \
-       time_t entry_time = time (NULL); \
-       int rc; \
-       do { \
-           ptr = x; \
-           if(!sess->api_block_mode || \
-              (ptr != NULL) || \
-              (libssh2_session_last_errno(sess) != LIBSSH2_ERROR_EAGAIN) ) \
-               break; \
-           rc = _libssh2_wait_socket(sess, entry_time); \
+        time_t entry_time = time(NULL); \
+        int rc; \
+        do { \
+            ptr = x; \
+            if(!sess->api_block_mode || \
+               (ptr != NULL) || \
+               (libssh2_session_last_errno(sess) != LIBSSH2_ERROR_EAGAIN)) \
+                break; \
+            rc = _libssh2_wait_socket(sess, entry_time); \
         } while(!rc); \
     } while(0)
 
@@ -90,4 +89,4 @@ int _libssh2_wait_socket(LIBSSH2_SESSION *session, time_t entry_time);
 /* this is the lib-internal set blocking function */
 int _libssh2_session_set_blocking(LIBSSH2_SESSION * session, int blocking);
 
-#endif /* LIBSSH2_SESSION_H */
+#endif /* __LIBSSH2_SESSION_H */
